@@ -1,12 +1,16 @@
 /***************************************************************************
-* Copyright (c) Johan Mabille, Sylvain Corlay, Wolf Vollprecht and         *
-* Martin Renou                                                             *
-* Copyright (c) QuantStack                                                 *
-*                                                                          *
-* Distributed under the terms of the BSD 3-Clause License.                 *
-*                                                                          *
-* The full license is in the file LICENSE, distributed with this software. *
-****************************************************************************/
+ * Copyright (c) Johan Mabille, Sylvain Corlay, Wolf Vollprecht and         *
+ * Martin Renou                                                             *
+ * Copyright (c) QuantStack                                                 *
+ * Copyright (c) Serge Guelton                                              *
+ *                                                                          *
+ * Distributed under the terms of the BSD 3-Clause License.                 *
+ *                                                                          *
+ * The full license is in the file LICENSE, distributed with this software. *
+ ****************************************************************************/
+
+#include "xsimd/xsimd.hpp"
+#ifndef XSIMD_NO_SUPPORTED_ARCHITECTURE
 
 #include <cmath>
 #include <functional>
@@ -20,8 +24,8 @@ template <class B>
 class batch_complex_test : public testing::Test
 {
 protected:
-
-    using batch_type = B;
+    using batch_type = xsimd::simd_type<typename B::value_type>;
+    using arch_type = typename B::arch_type;
     using real_batch_type = typename B::real_batch;
     using value_type = typename B::value_type;
     using real_value_type = typename value_type::value_type;
@@ -35,6 +39,11 @@ protected:
     value_type scalar;
     real_value_type real_scalar;
 
+#ifdef XSIMD_ENABLE_XTL_COMPLEX
+    using xtl_value_type = xtl::xcomplex<real_value_type, real_value_type, true>;
+    using xtl_array_type = std::array<xtl_value_type, size>;
+#endif
+
     batch_complex_test()
     {
         scalar = value_type(real_value_type(1.4), real_value_type(2.3));
@@ -51,17 +60,17 @@ protected:
     {
         {
             array_type res;
-            batch_type b;
-            b.load_unaligned(lhs.data());
+            batch_type b = batch_type::load_unaligned(lhs.data());
             b.store_unaligned(res.data());
             EXPECT_EQ(res, lhs) << print_function_name("load_unaligned / store_unaligned complex*");
 
-            alignas(XSIMD_DEFAULT_ALIGNMENT) array_type arhs(this->rhs);
-            alignas(XSIMD_DEFAULT_ALIGNMENT) array_type ares;
-            b.load_aligned(arhs.data());
+            alignas(arch_type::alignment()) array_type arhs(this->rhs);
+            alignas(arch_type::alignment()) array_type ares;
+            b = batch_type::load_aligned(arhs.data());
             b.store_aligned(ares.data());
             EXPECT_EQ(ares, rhs) << print_function_name("load_aligned / store_aligned complex*");
         }
+
         {
             real_array_type real, imag, res_real, res_imag;
             for (size_t i = 0; i < size; ++i)
@@ -69,42 +78,53 @@ protected:
                 real[i] = lhs[i].real();
                 imag[i] = lhs[i].imag();
             }
-            batch_type b;
-            b.load_unaligned(real.data(), imag.data());
+            batch_type b = batch_type::load_unaligned(real.data(), imag.data());
             b.store_unaligned(res_real.data(), res_imag.data());
             EXPECT_EQ(res_real, real) << print_function_name("load_unaligned / store_unaligned (real*, real*)");
 
-            alignas(XSIMD_DEFAULT_ALIGNMENT) real_array_type areal, aimag, ares_real, ares_imag;
+            alignas(arch_type::alignment()) real_array_type areal, aimag, ares_real, ares_imag;
             for (size_t i = 0; i < size; ++i)
             {
                 areal[i] = lhs[i].real();
                 aimag[i] = lhs[i].imag();
             }
-            b.load_aligned(areal.data(), aimag.data());
+            b = batch_type::load_aligned(areal.data(), aimag.data());
             b.store_aligned(ares_real.data(), ares_imag.data());
             EXPECT_EQ(ares_real, areal) << print_function_name("load_aligned / store_aligned (real*, real*)");
         }
         {
-            real_array_type real, res_real;
+            real_array_type real, imag, res_real, res_imag;
             for (size_t i = 0; i < size; ++i)
             {
                 real[i] = lhs[i].real();
+                imag[i] = 0;
             }
-            batch_type b;
-            b.load_unaligned(real.data());
-            b.store_unaligned(res_real.data());
+            batch_type b = batch_type::load_unaligned(real.data());
+            b.store_unaligned(res_real.data(), res_imag.data());
             EXPECT_EQ(res_real, real) << print_function_name("load_unaligned / store_unaligned (real*)");
+            EXPECT_EQ(res_imag, imag) << print_function_name("load_unaligned / store_unaligned (real*)");
 
-            alignas(XSIMD_DEFAULT_ALIGNMENT) real_array_type areal, ares_real;
+            alignas(arch_type::alignment()) real_array_type areal, aimag, ares_real, ares_imag;
             for (size_t i = 0; i < size; ++i)
             {
                 areal[i] = lhs[i].real();
+                aimag[i] = 0;
             }
-            b.load_aligned(areal.data());
-            b.store_aligned(ares_real.data());
+            b = batch_type::load_aligned(areal.data());
+            b.store_aligned(ares_real.data(), ares_imag.data());
             EXPECT_EQ(ares_real, areal) << print_function_name("load_aligned / store_aligned (real*)");
+            EXPECT_EQ(ares_imag, aimag) << print_function_name("load_aligned / store_aligned (real*)");
         }
     }
+#ifdef XSIMD_ENABLE_XTL_COMPLEX
+    void test_load_store_xtl() const
+    {
+        xtl_array_type tmp;
+        std::fill(tmp.begin(), tmp.end(), xtl_value_type(2, 3));
+        batch_type b0(xtl_value_type(2, 3));
+        EXPECT_EQ(b0, tmp) << print_function_name("batch(value_type)");
+    }
+#endif
 
     void test_constructors() const
     {
@@ -124,18 +144,6 @@ protected:
             imag[i] = lhs[i].imag();
             tmp[i] = value_type(real[i]);
         }
-
-        batch_type b2(real.data());
-        EXPECT_EQ(b2, tmp) << print_function_name("batch(real_batch)");
-
-        batch_type b3(real.data(), imag.data());
-        EXPECT_EQ(b3, lhs) << print_function_name("batch(real_batch, real_batch)");
-
-        batch_type b4(real_batch_type(real.data()));
-        EXPECT_EQ(b4, tmp) << print_function_name("batch(real_ptr)");
-
-        batch_type b5(real_batch_type(real.data()), real_batch_type(imag.data()));
-        EXPECT_EQ(b5, lhs) << print_function_name("batch(real_ptr, real_ptr)");
     }
 
     void test_access_operator() const
@@ -143,7 +151,7 @@ protected:
         batch_type res = batch_lhs();
         for (size_t i = 0; i < size; ++i)
         {
-            EXPECT_EQ(res[i], lhs[i]) << print_function_name("operator[](") << i << ")";
+            EXPECT_EQ(res.get(i), lhs[i]) << print_function_name("get(") << i << ")";
         }
     }
 
@@ -178,11 +186,13 @@ protected:
             batch_type rres = scalar + batch_lhs();
             EXPECT_BATCH_EQ(rres, expected) << print_function_name("scalar + batch");
         }
+
         // batch + real_batch
         {
             array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), rhs.cbegin(), expected.begin(),
-                            [](const value_type& l, const value_type& r) { return l + r.real(); });
+                           [](const value_type& l, const value_type& r)
+                           { return l + r.real(); });
             batch_type lres = batch_lhs() + batch_rhs().real();
             EXPECT_BATCH_EQ(lres, expected) << print_function_name("batch + real_batch");
             batch_type rres = batch_rhs().real() + batch_lhs();
@@ -218,11 +228,13 @@ protected:
         {
             array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), rhs.cbegin(), expected.begin(),
-                            [](const value_type& l, const value_type& r) { return l - r.real(); });
+                           [](const value_type& l, const value_type& r)
+                           { return l - r.real(); });
             batch_type lres = batch_lhs() - batch_rhs().real();
             EXPECT_BATCH_EQ(lres, expected) << print_function_name("batch - real_batch");
             std::transform(lhs.cbegin(), lhs.cend(), rhs.cbegin(), expected.begin(),
-                            [](const value_type& l, const value_type& r) { return r.real() - l; });
+                           [](const value_type& l, const value_type& r)
+                           { return r.real() - l; });
             batch_type rres = batch_rhs().real() - batch_lhs();
             EXPECT_BATCH_EQ(rres, expected) << print_function_name("real_batch - batch");
         }
@@ -256,7 +268,8 @@ protected:
         {
             array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), rhs.cbegin(), expected.begin(),
-                            [](const value_type& l, const value_type& r) { return l * r.real(); });
+                           [](const value_type& l, const value_type& r)
+                           { return l * r.real(); });
             batch_type lres = batch_lhs() * batch_rhs().real();
             EXPECT_BATCH_EQ(lres, expected) << print_function_name("batch * real_batch");
             batch_type rres = batch_rhs().real() * batch_lhs();
@@ -292,11 +305,13 @@ protected:
         {
             array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), rhs.cbegin(), expected.begin(),
-                            [](const value_type& l, const value_type& r) { return l / r.real(); });
+                           [](const value_type& l, const value_type& r)
+                           { return l / r.real(); });
             batch_type lres = batch_lhs() / batch_rhs().real();
             EXPECT_BATCH_EQ(lres, expected) << print_function_name("batch / real_batch");
             std::transform(lhs.cbegin(), lhs.cend(), rhs.cbegin(), expected.begin(),
-                            [](const value_type& l, const value_type& r) { return r.real() / l; });
+                           [](const value_type& l, const value_type& r)
+                           { return r.real() / l; });
             batch_type rres = batch_rhs().real() / batch_lhs();
             EXPECT_BATCH_EQ(rres, expected) << print_function_name("real_batch / batch");
         }
@@ -314,6 +329,7 @@ protected:
 
     void test_computed_assignment() const
     {
+
         // batch += batch
         {
             array_type expected;
@@ -326,7 +342,7 @@ protected:
         {
             array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), expected.begin(), std::bind(std::plus<value_type>(), _1, scalar));
-            batch_type res = batch_lhs(); 
+            batch_type res = batch_lhs();
             res += scalar;
             EXPECT_BATCH_EQ(res, expected) << print_function_name("batch += scalar");
         }
@@ -334,7 +350,8 @@ protected:
         {
             array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), rhs.cbegin(), expected.begin(),
-                            [](const value_type& l, const value_type& r) { return l + r.real(); });
+                           [](const value_type& l, const value_type& r)
+                           { return l + r.real(); });
             batch_type res = batch_lhs();
             res += batch_rhs().real();
             EXPECT_BATCH_EQ(res, expected) << print_function_name("batch += real_batch");
@@ -343,7 +360,7 @@ protected:
         {
             array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), expected.begin(), std::bind(std::plus<value_type>(), _1, real_scalar));
-            batch_type res = batch_lhs(); 
+            batch_type res = batch_lhs();
             res += real_scalar;
             EXPECT_BATCH_EQ(res, expected) << print_function_name("batch += real_scalar");
         }
@@ -367,7 +384,8 @@ protected:
         {
             array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), rhs.cbegin(), expected.begin(),
-                            [](const value_type& l, const value_type& r) { return l - r.real(); });
+                           [](const value_type& l, const value_type& r)
+                           { return l - r.real(); });
             batch_type res = batch_lhs();
             res -= batch_rhs().real();
             EXPECT_BATCH_EQ(res, expected) << print_function_name("batch -= real_batch");
@@ -376,7 +394,7 @@ protected:
         {
             array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), expected.begin(), std::bind(std::minus<value_type>(), _1, real_scalar));
-            batch_type res = batch_lhs(); 
+            batch_type res = batch_lhs();
             res -= real_scalar;
             EXPECT_BATCH_EQ(res, expected) << print_function_name("batch -= real_scalar");
         }
@@ -400,7 +418,8 @@ protected:
         {
             array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), rhs.cbegin(), expected.begin(),
-                            [](const value_type& l, const value_type& r) { return l * r.real(); });
+                           [](const value_type& l, const value_type& r)
+                           { return l * r.real(); });
             batch_type res = batch_lhs();
             res *= batch_rhs().real();
             EXPECT_BATCH_EQ(res, expected) << print_function_name("batch *= real_batch");
@@ -409,7 +428,7 @@ protected:
         {
             array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), expected.begin(), std::bind(std::multiplies<value_type>(), _1, real_scalar));
-            batch_type res = batch_lhs(); 
+            batch_type res = batch_lhs();
             res *= real_scalar;
             EXPECT_BATCH_EQ(res, expected) << print_function_name("batch *= real_scalar");
         }
@@ -433,7 +452,8 @@ protected:
         {
             array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), rhs.cbegin(), expected.begin(),
-                            [](const value_type& l, const value_type& r) { return l / r.real(); });
+                           [](const value_type& l, const value_type& r)
+                           { return l / r.real(); });
             batch_type res = batch_lhs();
             res /= batch_rhs().real();
             EXPECT_BATCH_EQ(res, expected) << print_function_name("batch /= real_batch");
@@ -442,7 +462,7 @@ protected:
         {
             array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), expected.begin(), std::bind(std::divides<value_type>(), _1, real_scalar));
-            batch_type res = batch_lhs(); 
+            batch_type res = batch_lhs();
             res /= real_scalar;
             EXPECT_BATCH_EQ(res, expected) << print_function_name("batch /= real_scalar");
         }
@@ -454,7 +474,8 @@ protected:
         {
             array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), expected.begin(),
-                            [](const value_type& v) { using std::conj; return conj(v); });
+                           [](const value_type& v)
+                           { using std::conj; return conj(v); });
             batch_type res = conj(batch_lhs());
             EXPECT_BATCH_EQ(res, expected) << print_function_name("conj");
         }
@@ -462,7 +483,8 @@ protected:
         {
             real_array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), expected.begin(),
-                            [](const value_type& v) { using std::norm; return norm(v); });
+                           [](const value_type& v)
+                           { using std::norm; return norm(v); });
             real_batch_type res = norm(batch_lhs());
             EXPECT_BATCH_EQ(res, expected) << print_function_name("norm");
         }
@@ -470,12 +492,44 @@ protected:
         {
             array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), expected.begin(),
-                            [](const value_type& v) { using std::proj; return proj(v); });
+                           [](const value_type& v)
+                           { using std::proj; return proj(v); });
             batch_type res = proj(batch_lhs());
             EXPECT_BATCH_EQ(res, expected) << print_function_name("proj");
         }
     }
 
+    void test_conj_norm_proj_real() const
+    {
+        // conj real batch
+        {
+            array_type expected;
+            std::transform(lhs.cbegin(), lhs.cend(), expected.begin(),
+                           [](const value_type& v)
+                           { return std::conj(std::real(v)); });
+            batch_type res = conj(real(batch_lhs()));
+            EXPECT_BATCH_EQ(res, expected) << print_function_name("conj(real batch)");
+        }
+        // norm real batch
+        {
+            real_array_type expected;
+            std::transform(lhs.cbegin(), lhs.cend(), expected.begin(),
+                           [](const value_type& v)
+                           { return std::norm(std::real(v)); });
+            real_batch_type res = norm(real(batch_lhs()));
+            EXPECT_BATCH_EQ(res, expected) << print_function_name("norm(real_batch)");
+        }
+        // proj real batch
+        {
+            array_type expected;
+            std::transform(lhs.cbegin(), lhs.cend(), expected.begin(),
+                           [](const value_type& v)
+                           { return std::proj(std::real(v)); });
+            batch_type res = proj(real(batch_lhs()));
+            EXPECT_BATCH_EQ(res, expected) << print_function_name("proj(real_batch)");
+        }
+    }
+
     void test_horizontal_operations() const
     {
         // hadd
@@ -492,7 +546,8 @@ protected:
         {
             array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), rhs.begin(), expected.begin(),
-                            [](const value_type& l, const value_type& r) { return l * r + r; });
+                           [](const value_type& l, const value_type& r)
+                           { return l * r + r; });
             batch_type res = xsimd::fma(batch_lhs(), batch_rhs(), batch_rhs());
             EXPECT_BATCH_EQ(res, expected) << print_function_name("fma");
         }
@@ -500,15 +555,18 @@ protected:
         {
             array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), rhs.begin(), expected.begin(),
-                            [](const value_type& l, const value_type& r) { return l * r - r; });
+                           [](const value_type& l, const value_type& r)
+                           { return l * r - r; });
             batch_type res = fms(batch_lhs(), batch_rhs(), batch_rhs());
             EXPECT_BATCH_EQ(res, expected) << print_function_name("fms");
         }
+
         // fnma
         {
             array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), rhs.begin(), expected.begin(),
-                            [](const value_type& l, const value_type& r) { return -l * r + r; });
+                           [](const value_type& l, const value_type& r)
+                           { return -l * r + r; });
             batch_type res = fnma(batch_lhs(), batch_rhs(), batch_rhs());
             EXPECT_BATCH_EQ(res, expected) << print_function_name("fnma");
         }
@@ -516,25 +574,48 @@ protected:
         {
             array_type expected;
             std::transform(lhs.cbegin(), lhs.cend(), rhs.begin(), expected.begin(),
-                            [](const value_type& l, const value_type& r) { return -l * r - r; });
+                           [](const value_type& l, const value_type& r)
+                           { return -l * r - r; });
             batch_type res = fnms(batch_lhs(), batch_rhs(), batch_rhs());
             EXPECT_BATCH_EQ(res, expected) << print_function_name("fnms");
         }
     }
 
-private:
+    void test_boolean_conversion() const
+    {
+        // !batch
+        {
+            array_type expected;
+            std::transform(lhs.cbegin(), lhs.cend(), expected.begin(),
+                           [](const value_type& l)
+                           { return l == value_type(0); });
+            batch_type res = (batch_type)!batch_lhs();
+            EXPECT_BATCH_EQ(res, expected) << print_function_name("!batch");
+        }
+    }
 
+    void test_isnan() const
+    {
+        {
+            bool_array_type expected;
+            std::transform(lhs.cbegin(), lhs.cend(), expected.begin(),
+                           [](const value_type& l)
+                           { return std::isnan(l.real()) || std::isnan(l.imag()); });
+            typename batch_type::batch_bool_type res = isnan(batch_lhs());
+            EXPECT_BATCH_EQ(res, expected) << print_function_name("isnan");
+        }
+    }
+
+private:
     batch_type batch_lhs() const
     {
-        batch_type res;
-        res.load_unaligned(lhs.data());
+        batch_type res = batch_type::load_unaligned(lhs.data());
         return res;
     }
 
     batch_type batch_rhs() const
     {
-        batch_type res;
-        res.load_unaligned(rhs.data());
+        batch_type res = batch_type::load_unaligned(rhs.data());
         return res;
     }
 };
@@ -546,6 +627,13 @@ TYPED_TEST(batch_complex_test, load_store)
     this->test_load_store();
 }
 
+#ifdef XSIMD_ENABLE_XTL_COMPLEX
+TYPED_TEST(batch_complex_test, load_store_xtl)
+{
+    this->test_load_store_xtl();
+}
+#endif
+
 TYPED_TEST(batch_complex_test, constructors)
 {
     this->test_constructors();
@@ -571,6 +659,11 @@ TYPED_TEST(batch_complex_test, conj_norm_proj)
     this->test_conj_norm_proj();
 }
 
+TYPED_TEST(batch_complex_test, conj_norm_proj_real)
+{
+    this->test_conj_norm_proj_real();
+}
+
 TYPED_TEST(batch_complex_test, horizontal_operations)
 {
     this->test_horizontal_operations();
@@ -580,3 +673,14 @@ TYPED_TEST(batch_complex_test, fused_operations)
 {
     this->test_fused_operations();
 }
+
+TYPED_TEST(batch_complex_test, boolean_conversion)
+{
+    this->test_boolean_conversion();
+}
+
+TYPED_TEST(batch_complex_test, isnan)
+{
+    this->test_isnan();
+}
+#endif
