@@ -1,15 +1,18 @@
 /***************************************************************************
-* Copyright (c) Johan Mabille, Sylvain Corlay, Wolf Vollprecht and         *
-* Martin Renou                                                             *
-* Copyright (c) QuantStack                                                 *
-*                                                                          *
-* Distributed under the terms of the BSD 3-Clause License.                 *
-*                                                                          *
-* The full license is in the file LICENSE, distributed with this software. *
-****************************************************************************/
+ * Copyright (c) Johan Mabille, Sylvain Corlay, Wolf Vollprecht and         *
+ * Martin Renou                                                             *
+ * Copyright (c) QuantStack                                                 *
+ * Copyright (c) Serge Guelton                                              *
+ *                                                                          *
+ * Distributed under the terms of the BSD 3-Clause License.                 *
+ *                                                                          *
+ * The full license is in the file LICENSE, distributed with this software. *
+ ****************************************************************************/
 
 #include <array>
 #include <climits>
+#include <cmath>
+#include <complex>
 #include <limits>
 #include <sstream>
 #include <type_traits>
@@ -22,6 +25,24 @@
 #ifndef XSIMD_TEST_UTILS_HPP
 #define XSIMD_TEST_UTILS_HPP
 
+/**************************
+ * AppleClang workarounds *
+ *************************/
+
+// AppleClang is known for having precision issues
+// in the gamma function codegen. It's known to happen
+// between AVX and AVX2, but it also happens on SSE4.1
+// in GitHub Actions.
+// This also seems to happen in M1.
+struct precision_t
+{
+#if defined(__apple_build_version__) && (XSIMD_WITH_SSE4_1 || XSIMD_WITH_NEON64)
+    static constexpr size_t max = 8192;
+#else
+    static constexpr size_t max = 2048;
+#endif
+};
+
 /*******************
  * Pretty printers *
  *******************/
@@ -29,13 +50,12 @@
 class simd_test_names
 {
 public:
-
     template <class T>
     static std::string GetName(int)
     {
         using value_type = typename T::value_type;
         std::string prefix = "fallback_";
-#if XSIMD_X86_INSTR_SET >= XSIMD_X86_SSE2_VERSION
+#if XSIMD_WITH_SSE
         size_t register_size = T::size * sizeof(value_type) * CHAR_BIT;
         if (register_size == size_t(128))
         {
@@ -49,28 +69,70 @@ public:
         {
             prefix = "avx512_";
         }
-#elif XSIMD_ARM_INSTR_SET >= XSIMD_ARM7_NEON_VERSION
+#elif XSIMD_WITH_NEON
         size_t register_size = T::size * sizeof(value_type) * CHAR_BIT;
         if (register_size == size_t(128))
         {
             prefix = "arm_";
         }
 #endif
-        if (std::is_same<value_type, uint8_t>::value) { return prefix + "uint8_t"; }
-        if (std::is_same<value_type, int8_t>::value) { return prefix + "int8_t"; }
-        if (std::is_same<value_type, uint16_t>::value) { return prefix + "uint16_t"; }
-        if (std::is_same<value_type, int16_t>::value) { return prefix + "int16_t"; }
-        if (std::is_same<value_type, uint32_t>::value) { return prefix + "uint32_t"; }
-        if (std::is_same<value_type, int32_t>::value) { return prefix + "int32_t"; }
-        if (std::is_same<value_type, uint64_t>::value) { return prefix + "uint64_t"; }
-        if (std::is_same<value_type, int64_t>::value) { return prefix + "int64_t"; }
-        if (std::is_same<value_type, float>::value) { return prefix + "float"; }
-        if (std::is_same<value_type, double>::value) { return prefix + "double"; }
-        if (std::is_same<value_type, std::complex<float>>::value) { return prefix + "complex<float>"; }
-        if (std::is_same<value_type, std::complex<double>>::value) { return prefix + "complex<double>"; }
+        if (std::is_same<value_type, uint8_t>::value)
+        {
+            return prefix + "uint8_t";
+        }
+        if (std::is_same<value_type, int8_t>::value)
+        {
+            return prefix + "int8_t";
+        }
+        if (std::is_same<value_type, uint16_t>::value)
+        {
+            return prefix + "uint16_t";
+        }
+        if (std::is_same<value_type, int16_t>::value)
+        {
+            return prefix + "int16_t";
+        }
+        if (std::is_same<value_type, uint32_t>::value)
+        {
+            return prefix + "uint32_t";
+        }
+        if (std::is_same<value_type, int32_t>::value)
+        {
+            return prefix + "int32_t";
+        }
+        if (std::is_same<value_type, uint64_t>::value)
+        {
+            return prefix + "uint64_t";
+        }
+        if (std::is_same<value_type, int64_t>::value)
+        {
+            return prefix + "int64_t";
+        }
+        if (std::is_same<value_type, float>::value)
+        {
+            return prefix + "float";
+        }
+        if (std::is_same<value_type, double>::value)
+        {
+            return prefix + "double";
+        }
+        if (std::is_same<value_type, std::complex<float>>::value)
+        {
+            return prefix + "complex<float>";
+        }
+        if (std::is_same<value_type, std::complex<double>>::value)
+        {
+            return prefix + "complex<double>";
+        }
 #ifdef XSIMD_ENABLE_XTL_COMPLEX
-        if (std::is_same<value_type, xtl::xcomplex<float>>::value) { return prefix + "xcomplex<float>"; }
-        if (std::is_same<value_type, xtl::xcomplex<double>>::value) { return prefix + "xcomplex<double>"; }
+        if (std::is_same<value_type, xtl::xcomplex<float>>::value)
+        {
+            return prefix + "xcomplex<float>";
+        }
+        if (std::is_same<value_type, xtl::xcomplex<double>>::value)
+        {
+            return prefix + "xcomplex<double>";
+        }
 #endif
 
         return prefix + "unknow_type";
@@ -88,19 +150,34 @@ inline std::string print_function_name(const std::string& func)
 
 namespace xsimd
 {
-    template <class T, size_t N>
-    inline bool operator==(const batch<T, N>& lhs, const std::array<T, N>& rhs)
+    template <class T, class A, size_t N>
+    inline bool operator==(const batch<T, A>& lhs, const std::array<T, N>& rhs)
     {
         std::array<T, N> tmp;
         lhs.store_unaligned(tmp.data());
         return tmp == rhs;
     }
 
-    template <class T, size_t N>
-    inline bool operator==(const std::array<T, N>& lhs, const batch<T, N>& rhs)
+    template <class T, class A, size_t N>
+    inline bool operator==(const std::array<T, N>& lhs, const batch<T, A>& rhs)
     {
         return rhs == lhs;
     }
+#ifdef XSIMD_ENABLE_XTL_COMPLEX
+    template <class T, class A, size_t N, bool i3ec>
+    inline bool operator==(const batch<std::complex<T>, A>& lhs, const std::array<xtl::xcomplex<T, T, i3ec>, N>& rhs)
+    {
+        std::array<xtl::xcomplex<T, T, i3ec>, N> tmp;
+        lhs.store_unaligned(tmp.data());
+        return tmp == rhs;
+    }
+
+    template <class T, class A, size_t N, bool i3ec>
+    inline bool operator==(const std::array<xtl::xcomplex<T, T, i3ec>, N>& lhs, const batch<std::complex<T>, A>& rhs)
+    {
+        return rhs == lhs;
+    }
+#endif
 }
 
 namespace detail
@@ -174,13 +251,15 @@ namespace detail
     template <class T>
     T safe_division(const T& lhs, const T& rhs)
     {
+        if (rhs == T(0))
+        {
+            return (std::numeric_limits<T>::max)();
+        }
         if (rhs < static_cast<T>(1) && lhs > rhs * (std::numeric_limits<T>::max)())
         {
             return (std::numeric_limits<T>::max)();
         }
-        if ((lhs == static_cast<T>(0)) ||
-            (rhs > static_cast<T>(1) &&
-                lhs < rhs * (std::numeric_limits<T>::min)()))
+        if ((lhs == static_cast<T>(0)) || (rhs > static_cast<T>(1) && lhs < rhs * (std::numeric_limits<T>::min)()))
         {
             return static_cast<T>(0);
         }
@@ -203,8 +282,8 @@ namespace detail
     {
         static bool run(const T& lhs, const T& rhs)
         {
-            using std::max;
             using std::abs;
+            using std::max;
 
             // direct compare integers -- but need tolerance for inexact double conversion
             if (std::is_integral<T>::value && lhs < 10e6 && rhs < 10e6)
@@ -222,8 +301,8 @@ namespace detail
                 return utils::isinf(rhs) && (lhs * rhs > 0) /* same sign */;
             }
 
-            T relative_precision = 2048 * std::numeric_limits<T>::epsilon();
-            T absolute_zero_prox = 2048 * std::numeric_limits<T>::epsilon();
+            T relative_precision = precision_t::max * std::numeric_limits<T>::epsilon();
+            T absolute_zero_prox = precision_t::max * std::numeric_limits<T>::epsilon();
 
             if (max(uabs(lhs), uabs(rhs)) < T(1e-3))
             {
@@ -262,8 +341,7 @@ namespace detail
         static bool run(const std::complex<T>& lhs, const std::complex<T>& rhs)
         {
             using real_comparison = scalar_comparison<T>;
-            return real_comparison::run(lhs.real(), rhs.real()) &&
-                real_comparison::run(lhs.imag(), rhs.imag());
+            return real_comparison::run(lhs.real(), rhs.real()) && real_comparison::run(lhs.imag(), rhs.imag());
         }
     };
 
@@ -274,8 +352,7 @@ namespace detail
         static bool run(const xtl::xcomplex<T, T, i3ec>& lhs, const xtl::xcomplex<T, T, i3ec>& rhs)
         {
             using real_comparison = scalar_comparison<T>;
-            return real_comparison::run(lhs.real(), rhs.real()) &&
-                real_comparison::run(lhs.imag(), rhs.imag());
+            return real_comparison::run(lhs.real(), rhs.real()) && real_comparison::run(lhs.imag(), rhs.imag());
         }
     };
 #endif
@@ -366,10 +443,10 @@ namespace detail
         return expect_container_near(lhs_expression, rhs_expression, lhs, rhs);
     }
 
-    template <class T, size_t N>
+    template <class T, size_t N, class A>
     testing::AssertionResult expect_batch_near(const char* lhs_expression,
                                                const char* rhs_expression,
-                                               const ::xsimd::batch<T, N>& lhs,
+                                               const ::xsimd::batch<T, A>& lhs,
                                                const std::array<T, N>& rhs)
     {
         std::array<T, N> tmp;
@@ -377,33 +454,33 @@ namespace detail
         return expect_array_near(lhs_expression, rhs_expression, tmp, rhs);
     }
 
-    template <class T, size_t N>
+    template <class T, size_t N, class A>
     testing::AssertionResult expect_batch_near(const char* lhs_expression,
                                                const char* rhs_expression,
                                                const std::array<T, N>& lhs,
-                                               const ::xsimd::batch<T, N>& rhs)
+                                               const ::xsimd::batch<T, A>& rhs)
     {
         std::array<T, N> tmp;
         rhs.store_unaligned(tmp.data());
         return expect_array_near(lhs_expression, rhs_expression, lhs, tmp);
-
     }
 
-    template <class T, size_t N>
+    template <class T, class A>
     testing::AssertionResult expect_batch_near(const char* lhs_expression,
                                                const char* rhs_expression,
-                                               const ::xsimd::batch<T, N>& lhs,
-                                               const ::xsimd::batch<T, N>& rhs)
+                                               const ::xsimd::batch<T, A>& lhs,
+                                               const ::xsimd::batch<T, A>& rhs)
     {
+        constexpr auto N = xsimd::batch<T, A>::size;
         std::array<T, N> tmp;
         lhs.store_unaligned(tmp.data());
         return expect_batch_near(lhs_expression, rhs_expression, tmp, rhs);
     }
 
-    template <class T, size_t N>
+    template <class T, size_t N, class A>
     testing::AssertionResult expect_batch_near(const char* lhs_expression,
                                                const char* rhs_expression,
-                                               const ::xsimd::batch_bool<T, N>& lhs,
+                                               const ::xsimd::batch_bool<T, A>& lhs,
                                                const std::array<bool, N>& rhs)
     {
         std::array<bool, N> tmp;
@@ -411,23 +488,24 @@ namespace detail
         return expect_array_near(lhs_expression, rhs_expression, tmp, rhs);
     }
 
-    template <class T, size_t N>
+    template <class T, size_t N, class A>
     testing::AssertionResult expect_batch_near(const char* lhs_expression,
                                                const char* rhs_expression,
                                                const std::array<bool, N>& lhs,
-                                               const ::xsimd::batch_bool<T, N>& rhs)
+                                               const ::xsimd::batch_bool<T, A>& rhs)
     {
         std::array<bool, N> tmp;
         rhs.store_unaligned(tmp.data());
         return expect_array_near(lhs_expression, rhs_expression, lhs, tmp);
     }
 
-    template <class T, size_t N>
+    template <class T, class A>
     testing::AssertionResult expect_batch_near(const char* lhs_expression,
                                                const char* rhs_expression,
-                                               const ::xsimd::batch_bool<T, N>& lhs,
-                                               const ::xsimd::batch_bool<T, N>& rhs)
+                                               const ::xsimd::batch_bool<T, A>& lhs,
+                                               const ::xsimd::batch_bool<T, A>& rhs)
     {
+        constexpr auto N = xsimd::batch<T, A>::size;
         std::array<bool, N> tmp;
         lhs.store_unaligned(tmp.data());
         return expect_batch_near(lhs_expression, rhs_expression, tmp, rhs);
@@ -460,10 +538,38 @@ namespace detail
         return get_nb_diff(lhs.begin(), lhs.end(), rhs.begin());
     }
 
+    template <class T, class A>
+    size_t get_nb_diff_near(const std::vector<T, A>& lhs, const std::vector<T, A>& rhs, float precision)
+    {
+        size_t i = 0;
+        for (size_t i = 0; i < lhs.size(); i++)
+        {
+            if (std::abs(lhs[i] - rhs[i]) > precision)
+            {
+                i++;
+            }
+        }
+        return i;
+    }
+
+    template <class T, size_t N>
+    size_t get_nb_diff_near(const std::array<T, N>& lhs, const std::array<T, N>& rhs, float precision)
+    {
+        size_t i = 0;
+        for (size_t i = 0; i < lhs.size(); i++)
+        {
+            if (std::abs(lhs[i] - rhs[i]) > precision)
+            {
+                i++;
+            }
+        }
+        return i;
+    }
+
     template <class B, class S>
     void load_batch(B& b, const S& src, size_t i = 0)
     {
-        b.load_unaligned(src.data() + i);
+        b = B::load_unaligned(src.data() + i);
     }
 
     template <class B, class D>
@@ -471,6 +577,16 @@ namespace detail
     {
         b.store_unaligned(dst.data() + i);
     }
+
+    inline xsimd::as_integer_t<float> nearbyint_as_int(float a)
+    {
+        return std::lroundf(a);
+    }
+
+    inline xsimd::as_integer_t<double> nearbyint_as_int(double a)
+    {
+        return std::llround(a);
+    }
 }
 
 #define EXPECT_BATCH_EQ(b1, b2) EXPECT_PRED_FORMAT2(::detail::expect_batch_near, b1, b2)
@@ -499,7 +615,9 @@ namespace xsimd
          * types_list *
          **************/
         template <class... T>
-        struct type_list {};
+        struct type_list
+        {
+        };
 
         /***************
          * concatenate *
@@ -551,153 +669,51 @@ using to_testing_types = xsimd::mpl::cast_t<T, testing::Types>;
 namespace xsimd
 {
     using batch_int_type_list = mpl::type_list<
-#if XSIMD_X86_INSTR_SET >= XSIMD_X86_SSE2_VERSION
-                                    batch<uint8_t, 16>,
-                                    batch<int8_t, 16>,
-                                    batch<uint16_t, 8>,
-                                    batch<int16_t, 8>,
-                                    batch<uint32_t, 4>,
-                                    batch<int32_t, 4>,
-                                    batch<uint64_t, 2>,
-                                    batch<int64_t, 2>
-#endif
-#if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX_VERSION
-                                    ,
-                                    batch<uint8_t, 32>,
-                                    batch<int8_t, 32>,
-                                    batch<uint16_t, 16>,
-                                    batch<int16_t, 16>,
-                                    batch<uint32_t, 8>,
-                                    batch<int32_t, 8>,
-                                    batch<uint64_t, 4>,
-                                    batch<int64_t, 4>
-#endif
-#if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX512_VERSION
-                                    ,
-                                    batch<uint8_t, 64>,
-                                    batch<int8_t, 64>,
-                                    batch<uint16_t, 32>,
-                                    batch<int16_t, 32>,
-                                    batch<uint32_t, 16>,
-                                    batch<int32_t, 16>,
-                                    batch<uint64_t, 8>,
-                                    batch<int64_t, 8>
-#endif
-#if XSIMD_ARM_INSTR_SET >= XSIMD_ARM7_NEON_VERSION
-                                    batch<uint8_t, 16>,
-                                    batch<int8_t, 16>,
-                                    batch<uint16_t, 8>,
-                                    batch<int16_t, 8>,
-                                    batch<uint32_t, 4>,
-                                    batch<int32_t, 4>,
-                                    batch<uint64_t, 2>,
-                                    batch<int64_t, 2>
-#endif
-#if defined(XSIMD_ENABLE_FALLBACK)
-                                    XSIMD_FALLBACK_DELIMITER
-                                    batch<int32_t, 7>,
-                                    batch<int64_t, 3>
-#endif
-                              >;
+        batch<uint8_t>,
+        batch<int8_t>,
+        batch<uint16_t>,
+        batch<int16_t>,
+        batch<uint32_t>,
+        batch<int32_t>,
+        batch<uint64_t>,
+        batch<int64_t>>;
 
-    using batch_float_type_list = mpl::type_list<
-#if XSIMD_X86_INSTR_SET >= XSIMD_X86_SSE2_VERSION
-                                     batch<float, 4>,
-                                     batch<double, 2>
+#if XSIMD_WITH_NEON && !XSIMD_WITH_NEON64
+    using batch_float_type_list = mpl::type_list<batch<float>>;
+#else
+    using batch_float_type_list = mpl::type_list<batch<float>, batch<double>>;
 #endif
-#if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX_VERSION
-                                     ,
-                                     batch<float, 8>,
-                                     batch<double, 4>
-#endif
-#if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX512_VERSION
-                                     ,
-                                     batch<float, 16>,
-                                     batch<double, 8>
-#endif
-#if XSIMD_ARM_INSTR_SET >= XSIMD_ARM7_NEON_VERSION
-                                     batch<float, 4>
-#endif
-#if XSIMD_ARM_INSTR_SET >= XSIMD_ARM8_64_NEON_VERSION
-                                     ,
-                                     batch<double, 2>
-#endif
-#if defined(XSIMD_ENABLE_FALLBACK)
-                                     XSIMD_FALLBACK_DELIMITER
-                                     batch<float, 7>,
-                                     batch<double, 3>
-#endif
-                                >;
 
     using batch_int32_type_list = mpl::type_list<
-#if XSIMD_X86_INSTR_SET >= XSIMD_X86_SSE2_VERSION
-                                    batch<int32_t, 4>
-#endif
-#if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX_VERSION
-                                    ,
-                                    batch<int32_t, 8>
-#endif
-#if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX512_VERSION
-                                    ,
-                                    batch<int32_t, 16>
-#endif
-#if XSIMD_ARM_INSTR_SET >= XSIMD_ARM7_NEON_VERSION
-                                    batch<int32_t, 4>
-#endif
-#if defined(XSIMD_ENABLE_FALLBACK)
-                                    XSIMD_FALLBACK_DELIMITER
-                                    batch<int32_t, 7>
-#endif
-    >;
+        batch<int32_t>>;
 
+#if XSIMD_WITH_NEON && !XSIMD_WITH_NEON64
     using batch_complex_type_list = mpl::type_list<
-#if XSIMD_X86_INSTR_SET >= XSIMD_X86_SSE2_VERSION
-                                    batch<std::complex<float>, 4>,
-                                    batch<std::complex<double>, 2>
-#ifdef XSIMD_ENABLE_XTL_COMPLEX
-                                    ,
-                                    batch<xtl::xcomplex<float>, 4>,
-                                    batch<xtl::xcomplex<double>, 2>
+        batch<std::complex<float>>>;
+#else
+    using batch_complex_type_list = mpl::type_list<
+        batch<std::complex<float>>,
+        batch<std::complex<double>>>;
 #endif
-#endif
-#if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX_VERSION
-                                    ,
-                                    batch<std::complex<float>, 8>,
-                                    batch<std::complex<double>, 4>
-#ifdef XSIMD_ENABLE_XTL_COMPLEX
-                                    ,
-                                    batch<xtl::xcomplex<float>, 8>,
-                                    batch<xtl::xcomplex<double>, 4>
-#endif
-#endif
-#if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX512_VERSION
-                                    ,
-                                    batch<std::complex<float>, 16>,
-                                    batch<std::complex<double>, 8>
-#ifdef XSIMD_ENABLE_XTL_COMPLEX
-                                    ,
-                                    batch<xtl::xcomplex<float>, 16>,
-                                    batch<xtl::xcomplex<double>, 8>
-#endif
-#endif
-#if XSIMD_ARM_INSTR_SET >= XSIMD_ARM7_NEON_VERSION
-                                     batch<std::complex<float>, 4>
-#ifdef XSIMD_ENABLE_XTL_COMPLEX
-                                     ,
-                                     batch<xtl::xcomplex<float>, 4>
-#endif
-#endif
-#if XSIMD_ARM_INSTR_SET >= XSIMD_ARM8_64_NEON_VERSION
-                                     ,
-                                     batch<std::complex<double>, 2>
-#ifdef XSIMD_ENABLE_XTL_COMPLEX
-                                     ,
-                                     batch<xtl::xcomplex<double>, 2>
-#endif
-#endif
-    >;
-
     using batch_math_type_list = mpl::concatenate_t<batch_int32_type_list, batch_float_type_list>;
+
+    using batch_swizzle_type_list = mpl::type_list<
+#if XSIMD_WITH_NEON64 || !XSIMD_WITH_NEON
+        batch<float>, batch<double>,
+#else
+        batch<float>,
+#endif
+#if !XSIMD_WITH_AVX || XSIMD_WITH_AVX2
+        batch<uint32_t>, batch<int32_t>,
+        batch<uint64_t>, batch<int64_t>,
+#endif
+        batch<std::complex<float>>
+#if XSIMD_WITH_NEON64 || !XSIMD_WITH_NEON
+        ,
+        batch<std::complex<double>>
+#endif
+        >;
+
     using batch_type_list = mpl::concatenate_t<batch_int_type_list, batch_float_type_list>;
 }
 
@@ -706,12 +722,11 @@ using batch_float_types = to_testing_types<xsimd::batch_float_type_list>;
 using batch_complex_types = to_testing_types<xsimd::batch_complex_type_list>;
 using batch_math_types = to_testing_types<xsimd::batch_math_type_list>;
 using batch_types = to_testing_types<xsimd::batch_type_list>;
-
+using batch_swizzle_types = to_testing_types<xsimd::batch_swizzle_type_list>;
 
 /********************
  * conversion utils *
  ********************/
-
 template <size_t N, size_t A>
 struct conversion_param
 {
@@ -722,45 +737,19 @@ struct conversion_param
 class conversion_test_names
 {
 public:
-
     template <class T>
     static std::string GetName(int)
     {
-#if XSIMD_X86_INSTR_SET >= XSIMD_X86_SSE2_VERSION
-        if (T::size == 2) { return "sse"; }
-        if (T::size == 4) { return "avx"; }
-        if (T::size == 8) { return "avx512";}
-        return "fallback";
-#elif XSIMD_ARM_INSTR_SET >= XSIMD_ARM7_NEON_VERSION
-        if (T::size == 2) { return "neon"; }
-        return "fallback";
+#ifndef _MSC_VER
+        return __PRETTY_FUNCTION__;
 #else
-        return "fallback";
+        return "Unknown name";
 #endif
     }
 };
 
 using conversion_type_list = xsimd::mpl::type_list<
-#if XSIMD_X86_INSTR_SET >= XSIMD_X86_SSE2_VERSION
-                               conversion_param<2, 16>
-#endif
-#if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX_VERSION
-                               ,
-                               conversion_param<4, 32>
-#endif
-#if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX512_VERSION
-                               ,
-                               conversion_param<8, 64>
-#endif
-#if XSIMD_ARM_INSTR_SET >= XSIMD_ARM8_64_NEON_VERSION
-                               conversion_param<2, 16>
-#endif
-#if defined(XSIMD_ENABLE_FALLBACK)
-                               XSIMD_FALLBACK_DELIMITER
-                               conversion_param<3, 32>
-#endif
-                               >;
+    conversion_param<sizeof(xsimd::types::simd_register<int, xsimd::default_arch>) / sizeof(double), xsimd::default_arch::alignment()>>;
 using conversion_types = to_testing_types<conversion_type_list>;
 
 #endif // XXSIMD_TEST_UTILS_HPP
-
